@@ -29,40 +29,49 @@
   #   }
   #   tempout=c(tempout, tempsel)
   # }
-  return=tempout
+  invisible(tempout)
 }
 
-profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, tolerance=4, ext=2, sigma=1, smooth=TRUE, SBlim, size=5, shape='disc', iters=6, threshold=1.05, converge='flux', magzero=0, gain=NULL, pixscale=1, sky, skyRMS, redosky=TRUE, redoskysize=21, box=c(100,100), grid=box, type='bilinear', skytype='median', skyRMStype='quanlo', sigmasel=1, doclip=TRUE, shiftloc = FALSE, paddim = TRUE, header, verbose=FALSE, plot=FALSE, stats=TRUE, rotstats=FALSE, boundstats=FALSE, nearstats=boundstats, groupstats=boundstats, offset=1, haralickstats=FALSE, sortcol="segID", decreasing=FALSE, lowmemory=FALSE, keepim=TRUE, R50clean=0, ...){
+profoundProFound=function(image=NULL, segim=NULL, objects=NULL, mask=NULL, skycut=1, pixcut=3, tolerance=4, ext=2, reltol=0, cliptol=Inf, sigma=1, smooth=TRUE, SBlim=NULL, size=5, shape='disc', iters=6, threshold=1.05, converge='flux', magzero=0, gain=NULL, pixscale=1, sky=NULL, skyRMS=NULL, redosegim=FALSE, redosky=TRUE, redoskysize=21, box=c(100,100), grid=box, type='bicubic', skytype='median', skyRMStype='quanlo', roughpedestal=FALSE, sigmasel=1, skypixmin=prod(box)/2, boxadd=box/2, boxiters=0, deblend=FALSE, df=3, radtrunc=2, iterative=FALSE, doclip=TRUE, shiftloc = FALSE, paddim = TRUE, header=NULL, verbose=FALSE, plot=FALSE, stats=TRUE, rotstats=FALSE, boundstats=FALSE, nearstats=boundstats, groupstats=boundstats, group=NULL, groupby='segim_orig', offset=1, haralickstats=FALSE, sortcol="segID", decreasing=FALSE, lowmemory=FALSE, keepim=TRUE, R50clean=0, watershed = 'EBImage', ...){
   if(verbose){message('Running ProFound:')}
   timestart=proc.time()[3]
+  
   call=match.call()
+  
   if(length(image)>1e6){rembig=TRUE}else{rembig=FALSE}
   
+  if(length(box)==1){
+    box=rep(box,2)
+    if(missing(grid)){grid=box}
+    if(missing(boxadd)){boxadd=box/2}
+    if(missing(skypixmin)){skypixmin=prod(box)/2}
+  }
+  
   #Split out image and header parts of input:
   
-  if(!missing(image)){
-    if(any(names(image)=='imDat') & missing(header)){
+  if(!is.null(image)){
+    if(any(names(image)=='imDat') & is.null(header)){
       if(verbose){message('Supplied image contains image and header components')}
       header=image$hdr
       image=image$imDat
-    }else if(any(names(image)=='imDat') & !missing(header)){
+    }else if(any(names(image)=='imDat') & !is.null(header)){
       if(verbose){message('Supplied image contains image and header but using specified header')}
       image=image$imDat
     }
-    if(any(names(image)=='dat') & missing(header)){
+    if(any(names(image)=='dat') & is.null(header)){
       if(verbose){message('Supplied image contains image and header components')}
       header=image$hdr[[1]]
       header=data.frame(key=header[,1],value=header[,2], stringsAsFactors = FALSE)
       image=image$dat[[1]]
-    }else if(any(names(image)=='dat') & !missing(header)){
+    }else if(any(names(image)=='dat') & !is.null(header)){
       if(verbose){message('Supplied image contains image and header but using specified header')}
       image=image$dat[[1]]
     }
-    if(any(names(image)=='image') & missing(header)){
+    if(any(names(image)=='image') & is.null(header)){
       if(verbose){message('Supplied image contains image and header components')}
       header=image$header
       image=image$image
-    }else if(any(names(image)=='image') & !missing(header)){
+    }else if(any(names(image)=='image') & !is.null(header)){
       if(verbose){message('Supplied image contains image and header but using specified header')}
       image=image$image
     }
@@ -72,7 +81,7 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
   
   #Treat image NAs as masked regions:
   
-  if(!missing(mask)){
+  if(!is.null(mask)){
     if(length(mask)==1){
       maskflag=mask
       mask=matrix(0L,dim(image)[1],dim(image)[2])
@@ -101,13 +110,13 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
     }
   }
   
-  #if(!missing(segim) & !missing(mask)){
+  #if(!is.null(segim) & !is.null(mask)){
   #  segim=segim*(1-mask) #I don't think we actually need this
   #}
   
   #Get the pixel scale, if possible and not provided:
   
-  if(missing(pixscale) & !missing(header)){
+  if(missing(pixscale) & !is.null(header)){
     pixscale=getpixscale(header)
     if(verbose){message(paste('Extracted pixel scale from header provided:',round(pixscale,3),'asec/pixel'))}
   }else{
@@ -117,8 +126,8 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
   skyarea=prod(dim(image))*pixscale^2/(3600^2)
   if(verbose){message(paste('Supplied image is',round(dim(image)[1]*pixscale/60,3),'x',round(dim(image)[2]*pixscale/60,3),'amin, ', round(skyarea,3),'deg-sq'))}
   
-  if(missing(objects)){
-    if(!missing(segim)){
+  if(is.null(objects)){
+    if(!is.null(segim)){
       objects=segim
       objects[objects != 0] = 1
       mode(objects)='integer'
@@ -127,21 +136,33 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
   
   #Check for user provided sky, and compute if missing:
   
-  hassky=!missing(sky)
-  hasskyRMS=!missing(skyRMS)
+  hassky=!is.null(sky)
+  hasskyRMS=!is.null(skyRMS)
   
-  if((hassky==FALSE | hasskyRMS==FALSE) & missing(segim)){
+  if((hassky==FALSE | hasskyRMS==FALSE) & is.null(segim)){
     if(verbose){message(paste('Making initial sky map -',round(proc.time()[3]-timestart,3),'sec'))}
-    roughsky=profoundMakeSkyGrid(image=image, objects=objects, mask=mask, box=box, grid=grid, type=type, shiftloc = shiftloc, paddim = paddim)
+    roughsky=profoundMakeSkyGrid(image=image, objects=objects, mask=mask, box=box, grid=grid, type=type, skytype=skytype, skyRMStype=skyRMStype, sigmasel=sigmasel, skypixmin=skypixmin, boxadd=boxadd, boxiters=0, doclip=doclip, shiftloc=shiftloc, paddim=paddim)
+    if(roughpedestal){
+      roughsky$sky=median(roughsky$sky)
+      roughsky$skyRMS=median(roughsky$skyRMS)
+    }
     if(hassky==FALSE){
       sky=roughsky$sky
-      if(verbose){message(' - Sky statistics :')}
-      if(verbose){print(summary(as.numeric(sky)))}
+      if(rembig==FALSE){
+        if(verbose){message(' - Sky statistics :')}
+        if(verbose){print(summary(as.numeric(sky)))}
+      }
     }
     if(hasskyRMS==FALSE){
       skyRMS=roughsky$skyRMS
-      if(verbose){message(' - Sky-RMS statistics :')}
-      if(verbose){print(summary(as.numeric(skyRMS)))}
+      if(rembig==FALSE){
+        if(verbose){message(' - Sky-RMS statistics :')}
+        if(verbose){print(summary(as.numeric(skyRMS)))}
+      }
+    }
+    if(rembig){
+      rm(roughsky)
+      gc()
     }
   }else{
     if(verbose){message("Skipping making initial sky map - User provided sky and sky RMS, or user provided segim")}
@@ -149,12 +170,13 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
   
   #Make the initial segmentation map, if not provided.
   
-  if(missing(segim)){
+  if(is.null(segim)){
     if(verbose){message(paste('Making initial segmentation image -',round(proc.time()[3]-timestart,3),'sec'))}
-    segim=profoundMakeSegim(image=image, objects=objects, mask=mask, tolerance=tolerance, ext=ext, sigma=sigma, smooth=smooth, pixcut=pixcut, skycut=skycut, SBlim=SBlim,  sky=sky, skyRMS=skyRMS, verbose=verbose, plot=FALSE, stats=FALSE)
+    segim=profoundMakeSegim(image=image, objects=objects, mask=mask, tolerance=tolerance, ext=ext, reltol=reltol, cliptol=cliptol, sigma=sigma, smooth=smooth, pixcut=pixcut, skycut=skycut, SBlim=SBlim,  sky=sky, skyRMS=skyRMS, verbose=verbose, watershed=watershed, plot=FALSE, stats=FALSE)
     objects=segim$objects
     segim=segim$segim
   }else{
+    redosegim=FALSE
     if(verbose){message("Skipping making an initial segmentation image - User provided segim")}
   }
   
@@ -163,7 +185,7 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
       if(verbose){message(paste('Doing initial aggressive dilation -',round(proc.time()[3]-timestart,3),'sec'))}
       objects_redo=profoundMakeSegimDilate(image=image, segim=objects, mask=mask, size=redoskysize, shape=shape, sky=sky, verbose=verbose, plot=FALSE, stats=FALSE, rotstats=FALSE)$objects
       if(verbose){message(paste('Making better sky map -',round(proc.time()[3]-timestart,3),'sec'))}
-      bettersky=profoundMakeSkyGrid(image=image, objects=objects_redo, mask=mask, box=box, grid=grid, type=type, shiftloc = shiftloc, paddim = paddim)
+      bettersky=profoundMakeSkyGrid(image=image, objects=objects_redo, mask=mask, box=box, grid=grid, type=type, skytype=skytype, skyRMStype=skyRMStype, sigmasel=sigmasel, skypixmin=skypixmin, boxadd=boxadd, boxiters=boxiters, doclip=doclip, shiftloc=shiftloc, paddim=paddim)
       if(hassky==FALSE){
         sky=bettersky$sky
         if(verbose){message(' - Sky statistics :')}
@@ -174,8 +196,23 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
         if(verbose){message(' - Sky-RMS statistics :')}
         if(verbose){print(summary(as.numeric(skyRMS)))}
       }
+      if(redosegim){
+        if(verbose){message(paste('Making better segmentation image -',round(proc.time()[3]-timestart,3),'sec'))}
+        imagescale=(image-sky)/skyRMS
+        imagescale[!is.finite(imagescale)]=0
+        if(!is.null(SBlim) & !missing(magzero)){
+          imagescale[imagescale<skycut | sky<profoundSB2Flux(SBlim, magzero, pixscale)]=0
+        }else{
+          imagescale[imagescale<skycut]=0
+        }
+        if(!is.null(mask)){
+          imagescale[mask!=0]=0
+        }
+        segim[imagescale==0]=0
+        objects[segim==0]=0
+      }
     }else{
-      if(verbose){message("Skipping making better sky map - User provided sky and sky RMS or iters=0")}
+      if(verbose){message("Skipping making better sky map - User provided sky and sky RMS")}
     }
     
     if(iters>0){
@@ -217,7 +254,7 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
         segim[select]=segim_array[,,i][select]
       }
       
-      if(!missing(mask)){
+      if(!is.null(mask)){
         segim[mask!=0]=segim_orig[mask!=0]
       }
       
@@ -247,7 +284,7 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
       if(verbose){message(paste('Doing final aggressive dilation -',round(proc.time()[3]-timestart,3),'sec'))}
       objects_redo=profoundMakeSegimDilate(image=image, segim=objects, mask=mask, size=redoskysize, shape=shape, sky=sky, verbose=verbose, plot=FALSE, stats=FALSE, rotstats=FALSE)$objects
       if(verbose){message(paste('Making final sky map -',round(proc.time()[3]-timestart,3),'sec'))}
-      sky=profoundMakeSkyGrid(image=image, objects=objects_redo, mask=mask, box=box, grid=grid, type=type, skytype=skytype, skyRMStype=skyRMStype, sigmasel=sigmasel, doclip=doclip, shiftloc = shiftloc, paddim = paddim)
+      sky=profoundMakeSkyGrid(image=image, objects=objects_redo, mask=mask, box=box, grid=grid, type=type, skytype=skytype, skyRMStype=skyRMStype, sigmasel=sigmasel, skypixmin=skypixmin, boxadd=boxadd, boxiters=boxiters, doclip=doclip, shiftloc=shiftloc, paddim=paddim)
       skyRMS=sky$skyRMS
       sky=sky$sky
       if(verbose){message(' - Sky statistics :')}
@@ -271,7 +308,7 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
       invisible(gc())
     }
     
-    if(stats & !missing(image)){
+    if(stats & !is.null(image)){
       if(verbose){message(paste('Calculating final segstats for',length(which(tabulate(segim)>0)),'objects -',round(proc.time()[3]-timestart,3),'sec'))}
       if(verbose){message(paste(' - magzero =', round(magzero,3)))}
       if(verbose){
@@ -298,19 +335,48 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
       near=NULL
     }
     
+    if(deblend){
+      groupstats=TRUE
+    }
+    
     if(groupstats){
-      group=profoundSegimGroup(segim=segim)
-      if(stats & !missing(image)){
+      if(verbose){message(' - groupstats = TRUE')}
+      if(groupby=='segim'){
+        if(is.null(group)){
+          group=profoundSegimGroup(segim)
+        }
+      }else if(groupby=='segim_orig'){
+        if(is.null(group)){
+          group=profoundSegimGroup(segim_orig)
+          if(any(group$groupsegID$Ngroup>1)){
+            group$groupim=profoundSegimKeep(segim=segim, segID_merge=group$groupsegID[group$groupsegID$Ngroup>1,'segID'])
+            group$groupsegID$Npix=tabulate(group$groupim)[group$groupsegID$groupID]
+          }
+        }
+      }else{
+        stop('Non legal groupby option, must be segim or segim_orig!')
+      }
+
+      if(stats & !is.null(image) & !is.null(group)){
         groupstats=profoundSegimStats(image=image, segim=group$groupim, mask=mask, sky=sky, skyRMS=skyRMS, magzero=magzero, gain=gain, pixscale=pixscale, header=header, sortcol=sortcol, decreasing=decreasing, rotstats=rotstats, boundstats=boundstats, offset=offset)
         colnames(groupstats)[1]='groupID'
       }else{
         groupstats=NULL
       }
     }else{
+      if(verbose){message(' - groupstats = FALSE')}
       group=NULL
       groupstats=NULL
     }
     
+    if(deblend & stats & !is.null(image) & any(group$groupsegID$Ngroup>1)){
+      if(verbose){message(' - deblend = TRUE')}
+      tempblend=profoundFluxDeblend(image=image-sky, segim=segim, segstats=segstats, groupim=group$groupim, groupsegID=group$groupsegID, magzero=magzero, df=df, radtrunc=radtrunc, iterative=iterative, doallstats=TRUE)
+      segstats=cbind(segstats,tempblend[,2:10])
+    }else{
+      if(verbose){message(' - deblend = FALSE')}
+    }
+    
     if(haralickstats){
       if(requireNamespace("EBImage", quietly = TRUE)){
         scale=10^(0.4*(30-magzero))
@@ -334,7 +400,7 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
         profoundSegimPlot(image=image, segim=segim, mask=mask, header=header, ...)
       }
     }else{
-      if(verbose){message("Skipping segmentation plot - plot set to FALSE")}
+      if(verbose){message("Skipping segmentation plot - plot = FALSE")}
     }
     
     if(!missing(SBlim)){
@@ -347,24 +413,24 @@ profoundProFound=function(image, segim, objects, mask, skycut=1, pixcut=3, toler
     }else{
       SBlim=NULL
     }
-    if(missing(header)){header=NULL}
+    if(is.null(header)){header=NULL}
     if(keepim==FALSE){image=NULL; mask=NULL}
-    if(missing(mask)){mask=NULL}
+    if(is.null(mask)){mask=NULL}
     if(verbose){message(paste('ProFound is finished! -',round(proc.time()[3]-timestart,3),'sec'))}
     output=list(segim=segim, segim_orig=segim_orig, objects=objects, objects_redo=objects_redo, sky=sky, skyRMS=skyRMS, image=image, mask=mask, segstats=segstats, Nseg=dim(segstats)[1], near=near, group=group, groupstats=groupstats, haralick=haralick, header=header, SBlim=SBlim, magzero=magzero, dim=dim(segim), pixscale=pixscale, skyarea=skyarea, gain=gain, call=call, date=date(), time=proc.time()[3]-timestart, ProFound.version=packageVersion('ProFound'), R.version=R.version)
   }else{
-    if(missing(header)){header=NULL}
+    if(is.null(header)){header=NULL}
     if(keepim==FALSE){image=NULL; mask=NULL}
-    if(missing(mask)){mask=NULL}
+    if(is.null(mask)){mask=NULL}
     if(verbose){message('No objects in segmentation map - skipping dilations and CoG')}
     if(verbose){message(paste('ProFound is finished! -',round(proc.time()[3]-timestart,3),'sec'))}
     output=list(segim=NULL, segim_orig=NULL, objects=NULL, objects_redo=NULL, sky=NULL, skyRMS=NULL, image=image, mask=mask, segstats=NULL, Nseg=0, near=NULL, group=NULL, groupstats=NULL, haralick=NULL, header=header, SBlim=NULL,  magzero=magzero, dim=dim(segim), pixscale=pixscale, skyarea=skyarea, gain=gain, call=call, date=date(), time=proc.time()[3]-timestart, ProFound.version=packageVersion('ProFound'), R.version=R.version)
   }
   class(output)='profound'
-  return=output
+  invisible(output)
 }
 
-plot.profound=function(x, logR50=TRUE, dmag=0.5, ...){
+plot.profound=function(x, logR50=TRUE, dmag=0.5, hist='sky', ...){
   
   if(class(x)!='profound'){
     stop('Object class is not of type profound!')
@@ -395,17 +461,17 @@ plot.profound=function(x, logR50=TRUE, dmag=0.5, ...){
   segdiff=x$segim-x$segim_orig
   segdiff[segdiff<0]=0
   
-  image=x$image-x$sky
+  image = (x$image-x$sky)/x$skyRMS
   cmap = rev(colorRampPalette(brewer.pal(9,'RdYlBu'))(100))
   maximg = quantile(abs(image), 0.995, na.rm=TRUE)
-  stretchscale = 1/median(abs(image[which(image>0)]), na.rm=TRUE)
+  stretchscale = 1/median(abs(image), na.rm=TRUE)
   
   layout(matrix(1:9, 3, byrow=TRUE))
   
   if(!is.null(x$header)){
   
     par(mar=c(3.5,3.5,0.5,0.5))
-    magimageWCS(image, x$header, stretchscale=stretchscale, locut=-maximg, hicut=maximg, type='num', zlim=c(0,1), col=cmap)
+    magimageWCS(image, x$header, stretchscale=stretchscale, locut=-maximg, hicut=maximg, range=c(-1,1), type='num', zlim=c(-1,1), col=cmap)
     if(!is.null(x$mask)){magimage(x$mask, locut=0, hicut=1, col=c(NA,hsv(v=0,alpha=0.2)), add=TRUE)}
     
     par(mar=c(3.5,3.5,0.5,0.5))
@@ -415,7 +481,7 @@ plot.profound=function(x, logR50=TRUE, dmag=0.5, ...){
     abline(h=c(0,dim(x$image)[2]))
     
     par(mar=c(3.5,3.5,0.5,0.5))
-    magimageWCS(image/x$skyRMS, x$header)
+    magimageWCS(image, x$header)
     magimage(segdiff, col=c(NA, rainbow(max(x$segim,na.rm=TRUE), end=2/3)), magmap=FALSE, add=TRUE)
     if(!is.null(x$mask)){magimage(x$mask, locut=0, hicut=1, col=c(NA,hsv(alpha=0.2)), add=TRUE)}
     
@@ -434,14 +500,29 @@ plot.profound=function(x, logR50=TRUE, dmag=0.5, ...){
     axis(side=1, at=xmax+0.25, labels=xmax+0.25, tick=FALSE, line=-1, col.axis='red')
       
     par(mar=c(3.5,3.5,0.5,0.5))
-    magimageWCS(x$sky, x$header)
+    stretchscale = 1/median(abs(x$sky), na.rm=TRUE)
+    magimageWCS(x$sky, x$header, locut=-max(abs(x$sky)), hicut=max(abs(x$sky)), range=c(-1,1), type='num', zlim=c(-1,1), stretchscale=stretchscale, col=cmap)
     legend('topleft',legend='sky',bg='white')
     
     par(mar=c(3.5,3.5,0.5,0.5))
     magimageWCS(x$skyRMS, x$header)
     legend('topleft',legend='skyRMS',bg='white')
     
-    maghist(x$segstats$iter, breaks=seq(-0.5,max(x$segstats$iter, na.rm=TRUE)+0.5,by=1), majorn=max(x$segstats$iter, na.rm=TRUE)+1, xlab='Number of Dilations', ylab='#')
+    if(hist=='iters'){
+      maghist(x$segstats$iter, breaks=seq(-0.5,max(x$segstats$iter, na.rm=TRUE)+0.5,by=1), majorn=max(x$segstats$iter, na.rm=TRUE)+1, xlab='Number of Dilations', ylab='#')
+    }else if(hist=='sky'){
+      try({
+        if(!is.null(x$objects_redo)){
+          tempsky=image[x$objects_redo==0]
+        }else{
+          tempsky=image[x$objects==0]
+        }
+        tempsky=tempsky[tempsky> -6 & tempsky<6]
+        magplot(density(tempsky, bw=0.1), grid=TRUE, xlim=c(-5,5), xlab='(image - sky) / skyRMS', ylab='PDF', log='y', ylim=c(1e-5,0.5))
+        curve(dnorm(x, mean=0, sd=1), add=TRUE, col='red', lty=2)
+        legend('topleft',legend='sky pixels',bg='white')
+        })
+    }else{stop('Not a recognised hist type! Must be iters / sky.')}
     
     par(mar=c(3.5,3.5,0.5,0.5))
     if(logR50){
@@ -457,7 +538,7 @@ plot.profound=function(x, logR50=TRUE, dmag=0.5, ...){
   }else{
     
     par(mar=c(3.5,3.5,0.5,0.5))
-    magimage(image, stretchscale=stretchscale, locut=-maximg, hicut=maximg, type='num', zlim=c(0,1), col=cmap)
+    magimage(image, stretchscale=stretchscale, locut=-maximg, hicut=maximg, range=c(-1,1), type='num', zlim=c(-1,1), col=cmap)
     if(!is.null(x$mask)){magimage(x$mask, locut=0, hicut=1, col=c(NA,hsv(v=0,alpha=0.2)), add=TRUE)}
     
     par(mar=c(3.5,3.5,0.5,0.5))
@@ -467,12 +548,12 @@ plot.profound=function(x, logR50=TRUE, dmag=0.5, ...){
     abline(h=c(0,dim(image)[2]))
     
     par(mar=c(3.5,3.5,0.5,0.5))
-    magimage(image/x$skyRMS)
+    magimage(image)
     magimage(segdiff, col=c(NA, rainbow(max(x$segim,na.rm=TRUE), end=2/3)), magmap=FALSE, add=TRUE)
     if(!is.null(x$mask)){magimage(x$mask, locut=0, hicut=1, col=c(NA,hsv(alpha=0.2)), add=TRUE)}
 
     par(mar=c(3.5,3.5,0.5,0.5))
-    temphist=maghist(x$segstats$mag, log='y', scale=(2*dmag), xlab='mag', ylab=paste('#/d',dmag,'mag',sep=''), grid=TRUE)
+    temphist=maghist(x$segstats$mag, log='y', scale=(2*dmag), breaks=seq(floor(min(x$segstats$mag, na.rm = TRUE)), ceiling(max(x$segstats$mag, na.rm = TRUE)),by=0.5), xlab='mag', ylab=paste('#/d',dmag,'mag',sep=''), grid=TRUE)
     ymax=log10(max(temphist$counts,na.rm = T))
     xmax=temphist$mids[which.max(temphist$counts)]
     abline(ymax - xmax*0.6, 0.6, col='red')
@@ -480,14 +561,29 @@ plot.profound=function(x, logR50=TRUE, dmag=0.5, ...){
     axis(side=1, at=xmax+0.25, labels=xmax+0.25, tick=FALSE, line=-1, col.axis='red')
     
     par(mar=c(3.5,3.5,0.5,0.5))
-    magimage(x$sky)
+    stretchscale = 1/median(abs(x$sky), na.rm=TRUE)
+    magimage(x$sky, locut=-max(abs(x$sky)), hicut=max(abs(x$sky)), range=c(-1,1), type='num', zlim=c(-1,1), col=cmap)
     legend('topleft',legend='sky',bg='white')
     
     par(mar=c(3.5,3.5,0.5,0.5))
     magimage(x$skyRMS)
     legend('topleft',legend='skyRMS',bg='white')
     
-    maghist(x$segstats$iter, breaks=seq(-0.5,max(x$segstats$iter, na.rm=TRUE)+0.5,by=1), majorn=max(x$segstats$iter, na.rm=TRUE)+1, xlab='Number of Dilations', ylab='#')
+    if(hist=='iters'){
+      maghist(x$segstats$iter, breaks=seq(-0.5,max(x$segstats$iter, na.rm=TRUE)+0.5,by=1), majorn=max(x$segstats$iter, na.rm=TRUE)+1, xlab='Number of Dilations', ylab='#')
+    }else if(hist=='sky'){
+      try({
+        if(!is.null(x$objects_redo)){
+          tempsky=image[x$objects_redo==0]
+        }else{
+          tempsky=image[x$objects==0]
+        }
+        tempsky=tempsky[tempsky> -6 & tempsky<6]
+        magplot(density(tempsky, bw=0.1), grid=TRUE, xlim=c(-5,5), xlab='(image - sky) / skyRMS', ylab='PDF', log='y', ylim=c(1e-5,0.5))
+        curve(dnorm(x, mean=0, sd=1), add=TRUE, col='red', lty=2)
+        legend('topleft',legend='sky pixels',bg='white')
+        })
+    }else{stop('Not a recognised hist type! Must be iters / sky.')}
     
     par(mar=c(3.5,3.5,0.5,0.5))
     if(logR50){
